@@ -5,7 +5,7 @@
 /*  Routines for figuring out the trigger date of a reminder   */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2020 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -24,6 +24,8 @@
 #define GOT_YR 4
 #define GOT_WD 8
 
+#define ADVANCE_TO_WD(x, wd) while (! ((wd) & (1 << ((x)%7)))) (x)++
+
 static int JYear(int jul);
 static int JMonth(int jul);
 static int NextSimpleTrig(int startdate, Trigger *trig, int *err);
@@ -58,9 +60,10 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
     if (trig->wd != NO_WD) typ |= GOT_WD;
     switch(typ) {
     case 0:
+        return startdate;
+
     case GOT_WD:
-	if (trig->wd != NO_WD)
-	    while(! (trig->wd & (1 << (startdate%7)))) startdate++;
+	ADVANCE_TO_WD(startdate, trig->wd);
 	return startdate;
 
     case GOT_DAY:
@@ -83,12 +86,12 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 	else return -1;
 
     case GOT_DAY+GOT_MON:
-	if (m > trig->m || (m == trig->m && d > trig->d)) y++;
 	if (trig->d > MonthDays[trig->m]) {
 	    *err = E_BAD_DATE;
 	    return -1;
 	}
 
+	if (m > trig->m || (m == trig->m && d > trig->d)) y++;
 	/* Take care of Feb. 29 */
 	while (trig->d > DaysInMonth(trig->m, y)) y++;
 	return Julian(y, trig->m, trig->d);
@@ -121,19 +124,19 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 	if (y > trig->y) return -1;
 	if (y < trig->y) j = Julian(trig->y, 0, 1);
 	else j = startdate;
-	while(! (trig->wd & (1 << (j%7)))) j++;
+        ADVANCE_TO_WD(j, trig->wd);
 	if (JYear(j) > trig->y) return -1;
 	return j;
 
     case GOT_MON+GOT_WD:
 	if (m == trig->m) {
 	    j = startdate;
-	    while(! (trig->wd & (1 << (j%7)))) j++;
+            ADVANCE_TO_WD(j, trig->wd);
 	    if (JMonth(j) == trig->m) return j;
 	}
 	if (m >= trig->m) j = Julian(y+1, trig->m, 1);
 	else j = Julian(y, trig->m, 1);
-	while(! (trig->wd & (1 << (j%7)))) j++;
+        ADVANCE_TO_WD(j, trig->wd);
 	return j; /* Guaranteed to be within the month */
 
     case GOT_DAY+GOT_WD:
@@ -144,7 +147,7 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 	    /* If there are fewer days in previous month, no match */
 	    if (trig->d <= DaysInMonth(m2, y2)) {
 		j = Julian(y2, m2, trig->d);
-		while(! (trig->wd & (1 << (j%7)))) j++;
+                ADVANCE_TO_WD(j, trig->wd);
 		if (j >= startdate) return j;
 
 	    }
@@ -153,7 +156,7 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 	/* Try this month */
 	if (trig->d <= DaysInMonth(m, y)) {
 	    j = Julian(y, m, trig->d);
-	    while(! (trig->wd & (1 << (j%7)))) j++;
+            ADVANCE_TO_WD(j, trig->wd);
 	    if (j >= startdate) return j;
 	}
 
@@ -162,18 +165,18 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 	if (m2 > 11) { m2 = 0; y++; }
 	while (trig->d > DaysInMonth(m2, y)) m2++;
 	j = Julian(y, m2, trig->d);
-	while(! (trig->wd & (1 << (j%7)))) j++;
+        ADVANCE_TO_WD(j, trig->wd);
 	return j;
 
     case GOT_WD+GOT_YR+GOT_DAY:
 	if (y > trig->y+1 || (y > trig->y && m>0)) return -1;
 	if (y > trig->y) {
 	    j = Julian(trig->y, 11, trig->d);
-	    while(! (trig->wd & (1 << (j%7)))) j++;
+            ADVANCE_TO_WD(j, trig->wd);
 	    if (j >= startdate) return j;
 	} else if (y < trig->y) {
 	    j = Julian(trig->y, 0, trig->d);
-	    while(! (trig->wd & (1 << (j%7)))) j++;
+            ADVANCE_TO_WD(j, trig->wd);
 	    return j;
 	} else {
 	    /* Try last month */
@@ -181,14 +184,16 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 		m2 = m-1;
 		while (trig->d > DaysInMonth(m2, trig->y)) m2--;
 		j = Julian(trig->y, m2, trig->d);
-		while(! (trig->wd & (1 << (j%7)))) j++;
+                ADVANCE_TO_WD(j, trig->wd);
+                if (JYear(j) > trig->y) return -1;
 		if (j >= startdate) return j;
 	    }
 	}
 	/* Try this month */
 	if (trig->d <= DaysInMonth(m, trig->y)) {
 	    j = Julian(trig->y, m, trig->d);
-	    while(! (trig->wd & (1 << (j%7)))) j++;
+            ADVANCE_TO_WD(j, trig->wd);
+            if (JYear(j) > trig->y) return -1;
 	    if (j >= startdate) return j;
 	}
 
@@ -197,7 +202,8 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 	m++;
 	while (trig->d > DaysInMonth(m, trig->d)) m++;
 	j = Julian(trig->y, m, trig->d);
-	while(! (trig->wd & (1 << (j%7)))) j++;
+        ADVANCE_TO_WD(j, trig->wd);
+        if (JYear(j) > trig->y) return -1;
 	return j;
 
     case GOT_DAY+GOT_MON+GOT_WD:
@@ -215,31 +221,32 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 
 	/* Try last year */
 	j = Julian(y, trig->m, trig->d);
-	while(! (trig->wd & (1 << (j%7)))) j++;
+        ADVANCE_TO_WD(j, trig->wd);
 	if (j >= startdate) return j;
 
 	/* Try this year */
 	y++;
+	while (trig->d > DaysInMonth(trig->m, y)) y++;
 	j = Julian(y, trig->m, trig->d);
-	while(! (trig->wd & (1 << (j%7)))) j++;
+        ADVANCE_TO_WD(j, trig->wd);
 	if (j >= startdate) return j;
 
 	/* Must be next year */
 	y++;
 	while (trig->d > DaysInMonth(trig->m, y)) y++;
 	j = Julian(y, trig->m, trig->d);
-	while(! (trig->wd & (1 << (j%7)))) j++;
+        ADVANCE_TO_WD(j, trig->wd);
 	return j;
 
     case GOT_WD+GOT_MON+GOT_YR:
 	if (y > trig->y || (y == trig->y && m > trig->m)) return -1;
 	if (trig->y > y || (trig->y == y && trig->m > m)) {
 	    j = Julian(trig->y, trig->m, 1);
-	    while(! (trig->wd & (1 << (j%7)))) j++;
+            ADVANCE_TO_WD(j, trig->wd);
 	    return j;
 	} else {
 	    j = startdate;
-	    while(! (trig->wd & (1 << (j%7)))) j++;
+            ADVANCE_TO_WD(j, trig->wd);
 	    FromJulian(j, &y2, &m2, &d2);
 	    if (m2 == trig->m) return j; else return -1;
 	}
@@ -250,7 +257,7 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 	    return -1;
 	}
 	j = Julian(trig->y, trig->m, trig->d);
-	while(! (trig->wd & (1 << (j%7)))) j++;
+        ADVANCE_TO_WD(j, trig->wd);
 	return j;
 
     default:
@@ -435,9 +442,9 @@ AdjustTriggerForDuration(int today, int r, Trigger *trig, TimeTrig *tim, int sav
 	    FromJulian(r, &y, &m, &d);
 	    fprintf(ErrFp, "%s(%d): Trig(adj) = %s, %d %s, %d",
 		    FileName, LineNo,
-		    DayName[r % 7],
+		    get_day_name(r % 7),
 		    d,
-		    MonthName[m],
+		    get_month_name(m),
 		    y);
 	    if (tim->ttime != NO_TIME) {
 		fprintf(ErrFp, " AT %02d:%02d",
@@ -586,9 +593,9 @@ int ComputeTriggerNoAdjustDuration(int today, Trigger *trig, TimeTrig *tim,
 		FromJulian(result, &y, &m, &d);
 		fprintf(ErrFp, "%s(%d): Trig = %s, %d %s, %d",
 			FileName, LineNo,
-			DayName[result % 7],
+			get_day_name(result % 7),
 			d,
-			MonthName[m],
+			get_month_name(m),
 			y);
 		if (tim->ttime != NO_TIME) {
 		    fprintf(ErrFp, " AT %02d:%02d",
