@@ -5,11 +5,15 @@
 /*  Useful utility functions.                                  */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2020 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
+static char const DontEscapeMe[] =
+"1234567890_-=+abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ@.,/";
+
 #include "config.h"
+#include "err.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -123,3 +127,38 @@ int DateOK(int y, int m, int d)
 	d > DaysInMonth(m, y) ) return 0;
     else return 1;
 }
+
+/* Functions designed to defeat gcc optimizer */
+
+int _private_div(int a, int b) { return a/b; }
+int _private_add_overflow(int result, int b, int old)
+{
+    if (b > 0 && result < old) return 1;
+    if (b < 0 && result > old) return 1;
+    return 0;
+}
+int _private_sub_overflow(int result, int b, int old)
+{
+    if (b < 0 && result < old) return 1;
+    if (b > 0 && result > old) return 1;
+    return 0;
+}
+
+int _private_unminus_overflow(int a, int b)
+{
+    if (a > 0 && b > 0) return 1;
+    if (a < 0 && b < 0) return 1;
+    return 0;
+}
+
+int
+ShellEscape(char const *in, DynamicBuffer *out)
+{
+    while(*in) {
+        if (!strchr(DontEscapeMe, *in)) {
+            if (DBufPutc(out, '\\') != OK) return E_NO_MEM;
+        }
+        if (DBufPutc(out, *in++) != OK) return E_NO_MEM;
+    }
+    return OK;
+}
