@@ -18,6 +18,7 @@
 
 #include <stdlib.h>
 #include <limits.h>
+#include <errno.h>
 #include "types.h"
 #include "expr.h"
 #include "globals.h"
@@ -39,6 +40,99 @@ static Var *VHashTbl[VAR_HASH_SIZE];
 
 typedef int (*SysVarFunc)(int, Value *);
 
+static void deprecated_var(char const *var, char const *instead)
+{
+    if (DebugFlag & DB_PRTLINE) {
+        Eprint("%s is deprecated; use %s instead", var, instead);
+        FreshLine = 1;
+    }
+}
+
+static int latlong_component_func(int do_set, Value *val, int *var, int min, int max, char const *varname, char const *newvarname)
+{
+    if (!do_set) {
+        val->type = INT_TYPE;
+        val->v.val = *var;
+        return OK;
+    }
+    deprecated_var(varname, newvarname);
+    if (val->type != INT_TYPE) return E_BAD_TYPE;
+    if (val->v.val < min) return E_2LOW;
+    if (val->v.val > max) return E_2HIGH;
+    *var = val->v.val;
+    set_lat_and_long_from_components();
+    return OK;
+}
+static int latdeg_func(int do_set, Value *val)
+{
+    return latlong_component_func(do_set, val, &LatDeg, -90, 90, "$LatDeg", "$Latitude");
+}
+
+static int latmin_func(int do_set, Value *val)
+{
+    return latlong_component_func(do_set, val, &LatMin, -59, 59, "$LatMin", "$Latitude");
+}
+
+static int latsec_func(int do_set, Value *val)
+{
+    return latlong_component_func(do_set, val, &LatSec, -59, 59, "$LatSec", "$Latitude");
+}
+
+static int longdeg_func(int do_set, Value *val)
+{
+    return latlong_component_func(do_set, val, &LongDeg, -180, 180, "$LongDeg", "$Longitude");
+}
+
+static int longmin_func(int do_set, Value *val)
+{
+    return latlong_component_func(do_set, val, &LongMin, -59, 59, "$LongMin", "$Longitude");
+}
+
+static int longsec_func(int do_set, Value *val)
+{
+    return latlong_component_func(do_set, val, &LongSec, -59, 59, "$LongSec", "$Longitude");
+}
+
+static int latitude_longitude_func(int do_set, Value *val, double *var, double min, double max) {
+    char buf[64];
+    double x;
+    char *endptr;
+
+    if (!do_set) {
+        snprintf(buf, sizeof(buf), "%f", *var);
+        val->v.str = malloc(strlen(buf)+1);
+        if (!val->v.str) return E_NO_MEM;
+        strcpy(val->v.str, buf);
+        val->type = STR_TYPE;
+        return OK;
+    }
+    if (val->type == INT_TYPE) {
+        x = (double) val->v.val;
+    } else {
+        if (val->type != STR_TYPE) return E_BAD_TYPE;
+        errno = 0;
+        x = strtod(val->v.str, &endptr);
+        if (errno) return E_BAD_TYPE;
+        if (*endptr) return E_BAD_TYPE;
+    }
+    if (x < min) return E_2LOW;
+    if (x > max) return E_2HIGH;
+    *var = x;
+    set_components_from_lat_and_long();
+    return OK;
+}
+
+static int longitude_func(int do_set, Value *val)
+{
+    return latitude_longitude_func(do_set, val, &Longitude, -180.0, 180.0);
+}
+
+static int latitude_func(int do_set, Value *val)
+{
+    return latitude_longitude_func(do_set, val, &Latitude, -90.0, 90.0);
+}
+
+
 static int trig_date_func(int do_set, Value *val)
 {
     UNUSED(do_set);
@@ -177,15 +271,15 @@ static int default_color_func(int do_set, Value *val)
     int col_r, col_g, col_b;
     if (!do_set) {
     /* 12 = strlen("255 255 255\0") */
- val->v.str = malloc(12);
- if (!val->v.str) return E_NO_MEM;
-    snprintf(val->v.str, 12, "%d %d %d",
-        DefaultColorR,
-        DefaultColorG,
-        DefaultColorB
-        );
- val->type = STR_TYPE;
- return OK;
+        val->v.str = malloc(12);
+        if (!val->v.str) return E_NO_MEM;
+        snprintf(val->v.str, 12, "%d %d %d",
+                 DefaultColorR,
+                 DefaultColorG,
+                 DefaultColorB
+            );
+        val->type = STR_TYPE;
+        return OK;
     }
     if (val->type != STR_TYPE) return E_BAD_TYPE;
     if (sscanf(val->v.str, "%d %d %d", &col_r, &col_g, &col_b) != 3) {
@@ -634,6 +728,7 @@ typedef struct {
     void *value;
     int min;
     int max;
+    int (*validate)(void const *newvalue);
 } SysVar;
 
 /* If the type of a sys variable is STR_TYPE, then min is redefined
@@ -645,61 +740,83 @@ typedef struct {
 
 /* All of the system variables sorted alphabetically */
 static SysVar SysVarArr[] = {
-    /*  name          mod  type              value         min/mal   max */
-    {"CalcUTC",        1,  INT_TYPE,     &CalculateUTC,       0,      1   },
-    {"CalMode",        0,  INT_TYPE,     &DoCalendar,         0,      0   },
-    {"Daemon",         0,  INT_TYPE,     &Daemon,             0,      0   },
-    {"DateSep",        1,  SPECIAL_TYPE, date_sep_func,       0,      0   },
-    {"DateTimeSep",    1,  SPECIAL_TYPE, datetime_sep_func,   0,      0   },
-    {"DefaultColor",   1,  SPECIAL_TYPE, default_color_func,  0,      0   },
-    {"DefaultPrio",    1,  INT_TYPE,     &DefaultPrio,        0,      9999},
-    {"DeltaOffset",    0,  INT_TYPE,     &DeltaOffset,        0,      0   },
-    {"DontFork",       0,  INT_TYPE,     &DontFork,           0,      0   },
-    {"DontQueue",      0,  INT_TYPE,     &DontQueue,          0,      0   },
-    {"DontTrigAts",    0,  INT_TYPE,     &DontIssueAts,       0,      0   },
-    {"EndSent",        1,  STR_TYPE,     &EndSent,            0,      0   },
-    {"EndSentIg",      1,  STR_TYPE,     &EndSentIg,          0,      0   },
-    {"FirstIndent",    1,  INT_TYPE,     &FirstIndent,        0,      132 },
-    {"FoldYear",       1,  INT_TYPE,     &FoldYear,           0,      1   },
-    {"FormWidth",      1,  INT_TYPE,     &FormWidth,          20,     500 },
-    {"HushMode",       0,  INT_TYPE,     &Hush,               0,      0   },
-    {"IgnoreOnce",     0,  INT_TYPE,     &IgnoreOnce,         0,      0   },
-    {"InfDelta",       0,  INT_TYPE,     &InfiniteDelta,      0,      0   },
-    {"IntMax",         0,  INT_TYPE,     &IntMax,             0,      0   },
-    {"IntMin",         0,  INT_TYPE,     &IntMin,             0,      0   },
-    {"LatDeg",         1,  INT_TYPE,     &LatDeg,             -90,    90  },
-    {"LatMin",         1,  INT_TYPE,     &LatMin,             -59,    59  },
-    {"LatSec",         1,  INT_TYPE,     &LatSec,             -59,    59  },
-    {"Location",       1,  STR_TYPE,     &Location,           0,      0   },
-    {"LongDeg",        1,  INT_TYPE,     &LongDeg,            -180,   180 },
-    {"LongMin",        1,  INT_TYPE,     &LongMin,            -59,    59  },
-    {"LongSec",        1,  INT_TYPE,     &LongSec,            -59,    59  },
-    {"MaxSatIter",     1,  INT_TYPE,     &MaxSatIter,         10,     ANY },
-    {"MaxStringLen",   1,  INT_TYPE,     &MaxStringLen,       -1,     ANY },
-    {"MinsFromUTC",    1,  INT_TYPE,     &MinsFromUTC,        -780,   780 },
-    {"NextMode",       0,  INT_TYPE,     &NextMode,           0,      0   },
-    {"NumQueued",      0,  INT_TYPE,     &NumQueued,          0,      0   },
-    {"NumTrig",        0,  INT_TYPE,     &NumTriggered,       0,      0   },
-    {"PrefixLineNo",   0,  INT_TYPE,     &DoPrefixLineNo,     0,      0   },
-    {"PSCal",          0,  INT_TYPE,     &PsCal,              0,      0   },
-    {"RunOff",         0,  INT_TYPE,     &RunDisabled,        0,      0   },
-    {"SimpleCal",      0,  INT_TYPE,     &DoSimpleCalendar,   0,      0   },
-    {"SortByDate",     0,  INT_TYPE,     &SortByDate,         0,      0   },
-    {"SortByPrio",     0,  INT_TYPE,     &SortByPrio,         0,      0   },
-    {"SortByTime",     0,  INT_TYPE,     &SortByTime,         0,      0   },
-    {"SubsIndent",     1,  INT_TYPE,     &SubsIndent,         0,      132 },
-    {"T",              0,  SPECIAL_TYPE, trig_date_func,      0,      0   },
-    {"Td",             0,  SPECIAL_TYPE, trig_day_func,       0,      0   },
-    {"TimeSep",        1,  SPECIAL_TYPE, time_sep_func,       0,      0   },
-    {"Tm",             0,  SPECIAL_TYPE, trig_mon_func,       0,      0   },
-    {"Tw",             0,  SPECIAL_TYPE, trig_wday_func,      0,      0   },
-    {"Ty",             0,  SPECIAL_TYPE, trig_year_func,      0,      0   },
-    {"U",              0,  SPECIAL_TYPE, today_date_func,     0,      0   },
-    {"Ud",             0,  SPECIAL_TYPE, today_day_func,      0,      0   },
-    {"Um",             0,  SPECIAL_TYPE, today_mon_func,      0,      0   },
-    {"UntimedFirst",   0,  INT_TYPE,     &UntimedBeforeTimed, 0,      0   },
-    {"Uw",             0,  SPECIAL_TYPE, today_wday_func,     0,      0   },
-    {"Uy",             0,  SPECIAL_TYPE, today_year_func,     0,      0   }
+    /*  name          mod  type              value          min/mal   max  validate*/
+    {"April",          1,  STR_TYPE,     &DynamicMonthName[3], 0,      0,    NULL },
+    {"August",         1,  STR_TYPE,     &DynamicMonthName[7], 0,      0,    NULL },
+    {"CalcUTC",        1,  INT_TYPE,     &CalculateUTC,        0,      1,    NULL },
+    {"CalMode",        0,  INT_TYPE,     &DoCalendar,          0,      0,    NULL },
+    {"Daemon",         0,  INT_TYPE,     &Daemon,              0,      0,    NULL },
+    {"DateSep",        1,  SPECIAL_TYPE, date_sep_func,        0,      0,    NULL },
+    {"DateTimeSep",    1,  SPECIAL_TYPE, datetime_sep_func,    0,      0,    NULL },
+    {"December",       1,  STR_TYPE,     &DynamicMonthName[11],0,      0,    NULL },
+    {"DefaultColor",   1,  SPECIAL_TYPE, default_color_func,   0,      0,    NULL },
+    {"DefaultPrio",    1,  INT_TYPE,     &DefaultPrio,         0,      9999, NULL },
+    {"DefaultTDelta",  1,  INT_TYPE,     &DefaultTDelta,       0,      1440, NULL },
+    {"DeltaOffset",    0,  INT_TYPE,     &DeltaOffset,         0,      0,    NULL },
+    {"DontFork",       0,  INT_TYPE,     &DontFork,            0,      0,    NULL },
+    {"DontQueue",      0,  INT_TYPE,     &DontQueue,           0,      0,    NULL },
+    {"DontTrigAts",    0,  INT_TYPE,     &DontIssueAts,        0,      0,    NULL },
+    {"EndSent",        1,  STR_TYPE,     &EndSent,             0,      0,    NULL },
+    {"EndSentIg",      1,  STR_TYPE,     &EndSentIg,           0,      0,    NULL },
+    {"February",       1,  STR_TYPE,     &DynamicMonthName[1], 0,      0,    NULL },
+    {"FirstIndent",    1,  INT_TYPE,     &FirstIndent,         0,      132,  NULL },
+    {"FoldYear",       1,  INT_TYPE,     &FoldYear,            0,      1,    NULL },
+    {"FormWidth",      1,  INT_TYPE,     &FormWidth,           20,     500,  NULL },
+    {"Friday",         1,  STR_TYPE,     &DynamicDayName[4],   0,      0,    NULL },
+    {"HushMode",       0,  INT_TYPE,     &Hush,                0,      0,    NULL },
+    {"IgnoreOnce",     0,  INT_TYPE,     &IgnoreOnce,          0,      0,    NULL },
+    {"InfDelta",       0,  INT_TYPE,     &InfiniteDelta,       0,      0,    NULL },
+    {"IntMax",         0,  INT_TYPE,     &IntMax,              0,      0,    NULL },
+    {"IntMin",         0,  INT_TYPE,     &IntMin,              0,      0,    NULL },
+    {"January",        1,  STR_TYPE,     &DynamicMonthName[0], 0,      0,    NULL },
+    {"July",           1,  STR_TYPE,     &DynamicMonthName[6], 0,      0,    NULL },
+    {"June",           1,  STR_TYPE,     &DynamicMonthName[5], 0,      0,    NULL },
+    {"LatDeg",         1,  SPECIAL_TYPE, latdeg_func,          0,      0,    NULL },
+    {"Latitude",       1,  SPECIAL_TYPE, latitude_func,        0,      0,    NULL },
+    {"LatMin",         1,  SPECIAL_TYPE, latmin_func,          0,      0,    NULL },
+    {"LatSec",         1,  SPECIAL_TYPE, latsec_func,          0,      0,    NULL },
+    {"Location",       1,  STR_TYPE,     &Location,            0,      0,    NULL },
+    {"LongDeg",        1,  SPECIAL_TYPE, longdeg_func,         0,      0,    NULL },
+    {"Longitude",      1,  SPECIAL_TYPE, longitude_func,       0,      0,    NULL },
+    {"LongMin",        1,  SPECIAL_TYPE, longmin_func,         0,      0,    NULL },
+    {"LongSec",        1,  SPECIAL_TYPE, longsec_func,         0,      0,    NULL },
+    {"March",          1,  STR_TYPE,     &DynamicMonthName[2], 0,      0,    NULL },
+    {"MaxSatIter",     1,  INT_TYPE,     &MaxSatIter,          10,     ANY,  NULL },
+    {"MaxStringLen",   1,  INT_TYPE,     &MaxStringLen,        -1,     ANY,  NULL },
+    {"May",            1,  STR_TYPE,     &DynamicMonthName[4], 0,      0,    NULL },
+    {"MinsFromUTC",    1,  INT_TYPE,     &MinsFromUTC,         -780,   780,  NULL },
+    {"Monday",         1,  STR_TYPE,     &DynamicDayName[0],   0,      0,    NULL },
+    {"NextMode",       0,  INT_TYPE,     &NextMode,            0,      0,    NULL },
+    {"November",       1,  STR_TYPE,     &DynamicMonthName[10],0,      0,    NULL },
+    {"NumQueued",      0,  INT_TYPE,     &NumQueued,           0,      0,    NULL },
+    {"NumTrig",        0,  INT_TYPE,     &NumTriggered,        0,      0,    NULL },
+    {"October",        1,  STR_TYPE,     &DynamicMonthName[9], 0,      0,    NULL },
+    {"PrefixLineNo",   0,  INT_TYPE,     &DoPrefixLineNo,      0,      0,    NULL },
+    {"PSCal",          0,  INT_TYPE,     &PsCal,               0,      0,    NULL },
+    {"RunOff",         0,  INT_TYPE,     &RunDisabled,         0,      0,    NULL },
+    {"Saturday",       1,  STR_TYPE,     &DynamicDayName[5],   0,      0,    NULL },
+    {"September",      1,  STR_TYPE,     &DynamicMonthName[8], 0,      0,    NULL },
+    {"SimpleCal",      0,  INT_TYPE,     &DoSimpleCalendar,    0,      0,    NULL },
+    {"SortByDate",     0,  INT_TYPE,     &SortByDate,          0,      0,    NULL },
+    {"SortByPrio",     0,  INT_TYPE,     &SortByPrio,          0,      0,    NULL },
+    {"SortByTime",     0,  INT_TYPE,     &SortByTime,          0,      0,    NULL },
+    {"SubsIndent",     1,  INT_TYPE,     &SubsIndent,          0,      132,  NULL },
+    {"Sunday",         1,  STR_TYPE,     &DynamicDayName[6],   0,      0,    NULL },
+    {"T",              0,  SPECIAL_TYPE, trig_date_func,       0,      0,    NULL },
+    {"Td",             0,  SPECIAL_TYPE, trig_day_func,        0,      0,    NULL },
+    {"Thursday",       1,  STR_TYPE,     &DynamicDayName[3],   0,      0,    NULL },
+    {"TimeSep",        1,  SPECIAL_TYPE, time_sep_func,        0,      0,    NULL },
+    {"Tm",             0,  SPECIAL_TYPE, trig_mon_func,        0,      0,    NULL },
+    {"Tuesday",        1,  STR_TYPE,     &DynamicDayName[1],   0,      0,    NULL },
+    {"Tw",             0,  SPECIAL_TYPE, trig_wday_func,       0,      0,    NULL },
+    {"Ty",             0,  SPECIAL_TYPE, trig_year_func,       0,      0,    NULL },
+    {"U",              0,  SPECIAL_TYPE, today_date_func,      0,      0,    NULL },
+    {"Ud",             0,  SPECIAL_TYPE, today_day_func,       0,      0,    NULL },
+    {"Um",             0,  SPECIAL_TYPE, today_mon_func,       0,      0,    NULL },
+    {"UntimedFirst",   0,  INT_TYPE,     &UntimedBeforeTimed,  0,      0,    NULL },
+    {"Uw",             0,  SPECIAL_TYPE, today_wday_func,      0,      0,    NULL },
+    {"Uy",             0,  SPECIAL_TYPE, today_year_func,      0,      0,    NULL },
+    {"Wednesday",      1,  STR_TYPE,     &DynamicDayName[2],   0,      0,    NULL }
 };
 
 #define NUMSYSVARS ( sizeof(SysVarArr) / sizeof(SysVar) )
@@ -714,6 +831,7 @@ static void DumpSysVar (char const *name, const SysVar *v);
 /***************************************************************/
 int SetSysVar(char const *name, Value *value)
 {
+    int r;
     SysVar *v = FindSysVar(name);
     if (!v) return E_NOSUCH_VAR;
     if (v->type != SPECIAL_TYPE &&
@@ -725,8 +843,27 @@ int SetSysVar(char const *name, Value *value)
 
     if (v->type == SPECIAL_TYPE) {
 	SysVarFunc f = (SysVarFunc) v->value;
-	return f(1, value);
-    } else if (v->type == STR_TYPE) {
+	r = f(1, value);
+        DestroyValue(*value);
+        return r;
+    }
+    if (v->validate) {
+        if (v->type == STR_TYPE) {
+            r = (v->validate)((void *) value->v.str);
+        } else {
+            r = (v->validate)((void *) &(value->v.val));
+        }
+        if (r != OK) {
+            return r;
+        }
+    }
+    if (v->type == STR_TYPE) {
+        /* If it's already the same, don't bother doing anything */
+        if (!strcmp(value->v.str, (char const *) v->value)) {
+            DestroyValue(*value);
+            return OK;
+        }
+
         /* If it's a string variable, special measures must be taken */
 	if (v->been_malloced) free(*((char **)(v->value)));
 	v->been_malloced = 1;
@@ -757,7 +894,11 @@ int GetSysVar(char const *name, Value *val)
 	SysVarFunc f = (SysVarFunc) v->value;
 	return f(0, val);
     } else if (v->type == STR_TYPE) {
-	val->v.str = StrDup(*((char **) v->value));
+        if (! * (char **) v->value) {
+            val->v.str = StrDup("");
+        } else {
+            val->v.str = StrDup(*((char **) v->value));
+        }
 	if (!val->v.str) return E_NO_MEM;
     } else {
 	val->v.val = *((int *) v->value);
@@ -767,7 +908,7 @@ int GetSysVar(char const *name, Value *val)
     /* In "verbose" mode, print attempts to test $RunOff */
     if (DebugFlag & DB_PRTLINE) {
 	if (v->value == (void *) &RunDisabled) {
-	    Eprint("(Security note: $RunOff variable tested.)\n");
+	    Eprint("(Security note: $RunOff variable tested.)");
 	    /* Allow further messages from this line */
 	    FreshLine = 1;
 	}
@@ -848,29 +989,29 @@ static void DumpSysVar(char const *name, const SysVar *v)
 	    SysVarFunc f = (SysVarFunc) v->value;
 	    f(0, &val);
 	    PrintValue(&val, ErrFp);
-	    Putc('\n', ErrFp);
+	    putc('\n', ErrFp);
 	    DestroyValue(val);
 	} else if (v->type == STR_TYPE) {
 	    char const *s = *((char **)v->value);
 	    int y;
-	    Putc('"', ErrFp);
+	    putc('"', ErrFp);
 	    for (y=0; y<MAX_PRT_LEN && *s; y++) {
 		if (*s == '"') {
 		    fprintf(ErrFp, "\" + char(34) + \"");
 		    s++;
 		} else {
-		    Putc(*s++, ErrFp);
+		    putc(*s++, ErrFp);
 		}
 	    }
-	    Putc('"', ErrFp);
+	    putc('"', ErrFp);
 	    if (*s) fprintf(ErrFp, "...");
-	    Putc('\n', ErrFp);
+	    putc('\n', ErrFp);
 	} else if (v->type == DATE_TYPE) {
 	    Value val;
 	    val.type = DATE_TYPE;
 	    val.v.val = * (int *) v->value;
 	    PrintValue(&val, ErrFp);
-	    Putc('\n', ErrFp);
+	    putc('\n', ErrFp);
 	} else {
 	    if (!v->modifiable) fprintf(ErrFp, "%d\n", *((int *)v->value));
 	    else {
@@ -885,3 +1026,48 @@ static void DumpSysVar(char const *name, const SysVar *v)
 
     return;
 }
+
+void
+set_lat_and_long_from_components(void)
+{
+    Latitude = (double) LatDeg + ((double) LatMin) / 60.0 + ((double) LatSec) / 3600.0;
+    Longitude = - ( (double) LongDeg + ((double) LongMin) / 60.0 + ((double) LongSec) / 3600.0);
+}
+
+void
+set_components_from_lat_and_long(void)
+{
+    double x;
+
+    x = (Latitude < 0.0 ? -Latitude : Latitude);
+    LatDeg = (int) x;
+    x -= (double) LatDeg;
+    x *= 60;
+    LatMin = (int) x;
+    x -= (double) LatMin;
+    x *= 60;
+    LatSec = (int) x;
+    if (Latitude < 0.0) {
+        LatDeg = -LatDeg;
+        LatMin = -LatMin;
+        LatSec = -LatSec;
+    }
+
+    x = (Longitude < 0.0 ? -Longitude : Longitude);
+    LongDeg = (int) x;
+    x -= (double) LongDeg;
+    x *= 60;
+    LongMin = (int) x;
+    x -= (double) LongMin;
+    x *= 60;
+    LongSec = (int) x;
+
+    /* Use STANDARD sign for $Longitude even if $LongDeg, $LongMin and
+     * $LongSec are messed up */
+    if (Longitude > 0.0) {
+        LongDeg = -LongDeg;
+        LongMin = -LongMin;
+        LongSec = -LongSec;
+    }
+}
+
